@@ -34,10 +34,18 @@
   (when (source-form-has-path-p form)
     (gethash form *source-paths*)))
 
+(defvar *transforming* nil)
+
 (defun ensure-source-path (form)
   (or (get-source-path form)
-      (cons (simplify-source-path-form form)
-            *current-path*)))
+      (if (and *transforming*
+               (not (memq 'transformed *current-path*)))
+          ;; Don't hide all the transformed paths, since we might want
+          ;; to look at the final form for error reporting.
+          (list* (simplify-source-path-form form)
+                 'transformed *current-path*)
+          (cons (simplify-source-path-form form)
+                *current-path*))))
 
 (defun simplify-source-path-form (form)
   (if (consp form)
@@ -316,36 +324,38 @@
 ;;; processed with MAKE-LOAD-FORM. We have to be careful, because
 ;;; CONSTANT might be circular. We also check that the constant (and
 ;;; any subparts) are dumpable at all.
-(defun maybe-emit-make-load-forms (constant &optional (name nil namep))
-  (let ((xset (alloc-xset)))
-    (labels ((grovel (value)
-               ;; Unless VALUE is an object which which obviously
-               ;; can't contain other objects
-               (unless (dumpable-leaflike-p value)
-                 (if (xset-member-p value xset)
-                     (return-from grovel nil)
-                     (add-to-xset value xset))
-                 (typecase value
-                   (cons
-                    (grovel (car value))
-                    (grovel (cdr value)))
-                   (simple-vector
-                    (dotimes (i (length value))
-                      (grovel (svref value i))))
-                   ((vector t)
-                    (dotimes (i (length value))
-                      (grovel (aref value i))))
-                   ((simple-array t)
+(defun ensure-externalizable (constant)
+  (declare (inline alloc-xset))
+  (dx-let ((xset (alloc-xset)))
+    (named-let grovel ((value constant))
+      ;; Unless VALUE is an object which which can't contain other objects,
+      ;; or was visited, or is acccessible in a named constant ...
+      (unless (or (dumpable-leaflike-p value)
+                  (xset-member-p value xset)
+                  (gethash value (eql-constants *ir1-namespace*)))
+        (add-to-xset value xset)
+        ;; FIXME: shouldn't this be something like SB-XC:TYPECASE ?
+        (typecase value
+          (cons
+           (grovel (car value))
+           (grovel (cdr value)))
+          (simple-vector
+           (dotimes (i (length value))
+             (grovel (svref value i))))
+          ((vector t)
+           (dotimes (i (length value))
+             (grovel (aref value i))))
+          ((simple-array t)
                     ;; Even though the (ARRAY T) branch does the exact
                     ;; same thing as this branch we do this separately
                     ;; so that the compiler can use faster versions of
                     ;; array-total-size and row-major-aref.
-                    (dotimes (i (array-total-size value))
-                      (grovel (row-major-aref value i))))
-                   ((array t)
-                    (dotimes (i (array-total-size value))
-                      (grovel (row-major-aref value i))))
-                   (instance
+           (dotimes (i (array-total-size value))
+             (grovel (row-major-aref value i))))
+          ((array t)
+           (dotimes (i (array-total-size value))
+             (grovel (row-major-aref value i))))
+          (instance
                     ;; In the target SBCL, we can dump any instance, but
                     ;; in the cross-compilation host, %INSTANCE-FOO
                     ;; functions don't work on general instances, only on
@@ -357,23 +367,36 @@
                     ;;
                     ;; FIXME: What about funcallable instances with
                     ;; user-defined MAKE-LOAD-FORM methods?
-                    (when (emit-make-load-form value)
-                      #+sb-xc-host
-                      (aver (eql (layout-bitmap (%instance-layout value))
-                                 sb-kernel:+layout-all-tagged+))
-                      (do-instance-tagged-slot (i value)
-                        (grovel (%instance-ref value i)))))
-                   (t
-                    (compiler-error
-                      "Objects of type ~/sb-impl:print-type-specifier/ ~
-                       can't be dumped into fasl files."
-                     (type-of value)))))))
-      ;; Dump all non-trivial named constants using the name.
-      (if (and namep (not (sb-xc:typep constant '(or symbol character fixnum
-                                                  #+64-bit single-float))))
-          (emit-make-load-form constant name)
-          (grovel constant))))
+           (when (emit-make-load-form value)
+             #+sb-xc-host
+             (aver (eql (layout-bitmap (%instance-layout value))
+                        sb-kernel:+layout-all-tagged+))
+             (do-instance-tagged-slot (i value)
+               (grovel (%instance-ref value i)))))
+          (t
+           (compiler-error
+            "Objects of type ~/sb-impl:print-type-specifier/ can't be dumped into fasl files."
+            (type-of value)))))))
   (values))
+
+;;; A constant is trivially externalizable if it involves no INSTANCE types
+;;; or any un-dumpable object.
+(defun trivially-externalizable-p (constant)
+  (declare (inline alloc-xset))
+  (dx-let ((xset (alloc-xset)))
+    (named-let ok ((value constant))
+      (if (or (dumpable-leaflike-p value) (xset-member-p value xset))
+          t
+          (progn
+            (add-to-xset value xset)
+            (typecase value
+             (cons (and (ok (car value)) (ok (cdr value))))
+             (vector
+              (dotimes (i (length value) t)
+                (unless (ok (aref value i)) (return nil))))
+             (array
+              (dotimes (i (array-total-size value) t)
+                (unless (ok (row-major-aref value i)) (return nil))))))))))
 
 ;;;; some flow-graph hacking utilities
 
@@ -1091,7 +1114,8 @@
                          (record-call name (ctran-block start) *current-path*))
                        (when *show-transforms-p*
                          (show-transform "src" name transformed))
-                       (ir1-convert start next result transformed))))
+                       (let ((*transforming* t))
+                         (ir1-convert start next result transformed)))))
               (ir1-convert-maybe-predicate start next result form var))))))
 
 ;;; KLUDGE: If we insert a synthetic IF for a function with the PREDICATE
@@ -1315,6 +1339,10 @@
       ;; to be bound... yet nowhere does it say that the special declaration
       ;; removes the constantness. Call it a spec bug and prohibit it. Same
       ;; for GLOBAL variables.
+      (unless (symbolp name)
+        (compiler-error
+         "~S is not a symbol and thus can't be declared special."
+         name))
       (let ((kind (info :variable :kind name)))
         (unless (member kind '(:special :unknown))
           (compiler-error
