@@ -459,31 +459,43 @@ necessary, since type inference may take arbitrarily long to converge.")
 (defparameter *reoptimize-limit* 10)
 
 (defun ir1-optimize-phase-1 (component)
-  (let ((loop-count 0))
-    (loop
-     (ir1-optimize-until-done component)
-     (when (or (component-new-functionals component)
-               (component-reanalyze-functionals component))
-       (maybe-mumble "Locall ")
-       (locall-analyze-component component))
-     (eliminate-dead-code component)
-     (dfo-as-needed component)
-     (when *constraint-propagate*
-       (maybe-mumble "Constraint ")
-       (constraint-propagate component))
-     (when (retry-delayed-ir1-transforms :constraint)
-       (setf loop-count 0) ;; otherwise nothing may get retried
-       (maybe-mumble "Rtran "))
-     (unless (or (component-reoptimize component)
-                 (component-reanalyze component)
-                 (component-new-functionals component)
-                 (component-reanalyze-functionals component))
-       (return))
-     (when (> loop-count *reoptimize-limit*)
-       (maybe-mumble "[Reoptimize Limit]")
-       (event reoptimize-maxed-out)
-       (return))
-     (incf loop-count))))
+  (let ((loop-count 0)
+        (*constraint-propagate* *constraint-propagate*))
+    (tagbody
+     again
+       (loop
+        (ir1-optimize-until-done component)
+        (when (or (component-new-functionals component)
+                  (component-reanalyze-functionals component))
+          (maybe-mumble "Locall ")
+          (locall-analyze-component component))
+        (eliminate-dead-code component)
+        (dfo-as-needed component)
+        (when *constraint-propagate*
+          (maybe-mumble "Constraint ")
+          (constraint-propagate component)
+          (when (retry-delayed-ir1-transforms :constraint)
+            (setf loop-count 0) ;; otherwise nothing may get retried
+            (maybe-mumble "Rtran ")))
+        (unless (or (component-reoptimize component)
+                    (component-reanalyze component)
+                    (component-new-functionals component)
+                    (component-reanalyze-functionals component))
+          (return))
+        (when (> loop-count *reoptimize-limit*)
+          (maybe-mumble "[Reoptimize Limit]")
+          (event reoptimize-maxed-out)
+          (return))
+        (incf loop-count))
+       ;; Do it once more for the transforms that will produce code
+       ;; that loses some information for further optimizations and
+       ;; it's better to insert it at the last moment.
+       ;; Such code shouldn't need constraint propagation, the slowest
+       ;; part, so avoid it.
+       (when (retry-delayed-ir1-transforms :ir1-phases)
+         (setf loop-count 0
+               *constraint-propagate* nil)
+         (go again)))))
 
 ;;; Do all the IR1 phases for a non-top-level component.
 (defun ir1-phases (component)
@@ -769,7 +781,7 @@ necessary, since type inference may take arbitrarily long to converge.")
   (maphash (lambda (k v)
              (declare (ignore k))
              (setf (leaf-info v) nil))
-           (eq-constants ns))
+           (eql-constants ns))
   (maphash (lambda (k v)
              (declare (ignore k))
              (when (constant-p v)
@@ -795,10 +807,10 @@ necessary, since type inference may take arbitrarily long to converge.")
              (eq (node-component x) component)))
     (blast (free-vars ns))
     (blast (free-funs ns))
-    ;; There can be more constants to blast when considering them by EQ rather
+    ;; There can be more constants to blast when considering them by EQL rather
     ;; than similarity. But it's totally OK to visit a #<CONSTANT> twice.
     ;; Its refs will be scanned redundantly, which is harmless.
-    (blast (eq-constants ns)))
+    (blast (eql-constants ns)))
   (values))
 
 ;;;; trace output
@@ -1953,8 +1965,7 @@ returning its filename.
 (defvar *constants-being-created* nil)
 (defvar *constants-created-since-last-init* nil)
 ;;; FIXME: Shouldn't these^ variables be unbound outside LET forms?
-(defun emit-make-load-form (constant &optional (name nil namep)
-                                     &aux (fasl *compile-object*))
+(defun emit-make-load-form (constant &aux (fasl *compile-object*))
   (aver (fasl-output-p fasl))
   (unless (fasl-constant-already-dumped-p constant fasl)
     (let ((circular-ref (assoc constant *constants-being-created* :test #'eq)))
@@ -1962,20 +1973,7 @@ returning its filename.
         (when (find constant *constants-created-since-last-init* :test #'eq)
           (throw constant t))
         (throw 'pending-init circular-ref)))
-    ;; If this is a global constant reference, we can call SYMBOL-GLOBAL-VALUE
-    ;; during LOAD as a fasl op, and not compile a lambda.
-    ;; However: the cross-compiler can not always emit fop-funcall for this,
-    ;; because the order of load-time actions is not strictly preserved as it
-    ;; would be for normal compilation. If the symbol's value needs computation,
-    ;; then it is unbound during genesis.
-    ;; So check if assignment was deferred, and if so, also defer the use.
-    (when (and namep #+sb-xc-host (not (member name *!const-value-deferred*)))
-      (fopcompile `(symbol-global-value ',name) nil t nil)
-      (fasl-note-handle-for-constant constant (sb-fasl::dump-pop fasl) fasl)
-      (return-from emit-make-load-form nil))
-    (multiple-value-bind (creation-form init-form)
-        (cond (namep (values `(symbol-global-value ',name) nil))
-              (t (%make-load-form constant)))
+    (multiple-value-bind (creation-form init-form) (%make-load-form constant)
       (cond
         ((eq init-form 'sb-fasl::fop-struct)
          (fasl-note-dumpable-instance constant fasl)
