@@ -1135,6 +1135,7 @@ default-value-8
 
 ;;; Turn more arg (context, count) into a list.
 (define-vop ()
+  (:translate %listify-rest-args)
   (:args (context-arg :target context :scs (descriptor-reg))
          (count-arg :target count :scs (any-reg)))
   (:arg-types * tagged-num)
@@ -1144,7 +1145,6 @@ default-value-8
   (:temporary (:scs (non-descriptor-reg) :from :eval) dst)
   (:temporary (:sc non-descriptor-reg :offset nl3-offset) pa-flag)
   (:results (result :scs (descriptor-reg)))
-  (:translate %listify-rest-args)
   (:policy :safe)
   (:node-var node)
   (:generator 20
@@ -1160,7 +1160,7 @@ default-value-8
       (inst beq done)
 
     ;; We need to do this atomically.
-    (pseudo-atomic (pa-flag)
+    (pseudo-atomic (pa-flag :sync nil)
       ;; Allocate a cons (2 words) for each item.
       (if dx-p
           (progn
