@@ -121,8 +121,6 @@
                           (find-class ',class) ,class)))
                classes)))
 
-(defun !wrapper-p (x) (and (sb-kernel::layout-p x) (layout-for-pcl-obj-p x)))
-
 (defun !bootstrap-meta-braid ()
   (let* ((*create-classes-from-internal-structure-definitions-p* nil)
          standard-class-wrapper standard-class
@@ -209,7 +207,7 @@
                   (error "Slot allocation ~S is not supported in bootstrap."
                          (getf slot :allocation))))
 
-              (when (!wrapper-p wrapper)
+              (when (layout-for-pcl-obj-p wrapper)
                 (setf (layout-slot-list wrapper) slots))
 
               (setq proto (if (eq meta 'funcallable-standard-class)
@@ -226,7 +224,7 @@
                      standard-effective-slot-definition-wrapper t))
 
               (setf (layout-slot-table wrapper) (make-slot-table class slots t))
-              (when (!wrapper-p wrapper)
+              (when (layout-for-pcl-obj-p wrapper)
                 (setf (layout-slot-list wrapper) slots))
 
               (case meta
@@ -359,7 +357,7 @@
             (make-slot-table class slots
                              (member metaclass-name
                                      '(standard-class funcallable-standard-class))))
-      (when (!wrapper-p wrapper)
+      (when (layout-for-pcl-obj-p wrapper)
         (setf (layout-slot-list wrapper) slots)))
 
     ;; For all direct superclasses SUPER of CLASS, make sure CLASS is
@@ -655,6 +653,36 @@
         (add-method gf class-method)))
     gf))
 
+(define-load-time-global *simple-stream-root-classoid* :unknown)
+
+(defun assign-layout-bitmap (layout)
+  ;; We decide only at class finalization time whether it is funcallable.
+  ;; Picking the right bitmap could probably be done sooner given the metaclass,
+  ;; but this approach avoids changing how PCL uses MAKE-LAYOUT.
+  ;; The big comment above MAKE-IMMOBILE-FUNINSTANCE in src/code/x86-64-vm
+  ;; explains why we differentiate between SGF and everything else.
+  (let ((inherits (layout-inherits layout)))
+    (when (find #.(find-layout 'stream) inherits)
+      (let ((flags 0))
+        (dovector (layout inherits)
+          (when (eq (layout-classoid layout) *simple-stream-root-classoid*)
+            (setq flags (logior flags +simple-stream-layout-flag+)))
+          (case layout
+            (#.(find-layout 'file-stream)
+             (setq flags (logior flags +file-stream-layout-flag+)))
+            (#.(find-layout 'string-stream)
+             (setq flags +string-stream-layout-flag+))))
+        (setf (layout-flags layout)
+              (logior flags +stream-layout-flag+ (layout-flags layout)))))
+    (when (find #.(find-layout 'function) inherits)
+      (setf (%raw-instance-ref/signed-word layout (sb-kernel::type-dd-length layout))
+             #+immobile-code ; there are two possible bitmaps
+             (if (or (find *sgf-wrapper* inherits) (eq layout *sgf-wrapper*))
+                 sb-kernel::standard-gf-primitive-obj-layout-bitmap
+                 +layout-all-tagged+)
+             ;; there is only one possible bitmap otherwise
+             #-immobile-code sb-kernel::standard-gf-primitive-obj-layout-bitmap))))
+
 ;;; Set the inherits from CPL, and register the layout. This actually
 ;;; installs the class in the Lisp type system.
 (defun %update-lisp-class-layout (class layout)
@@ -665,7 +693,8 @@
                            (order-layout-inherits
                             (map 'simple-vector #'class-wrapper
                                  (reverse (rest (class-precedence-list class)))))
-                           t)
+                           nil 0)
+      (assign-layout-bitmap layout)
       (register-layout layout :invalidate t)
 
       ;; FIXME: I don't think this should be necessary, but without it
