@@ -63,7 +63,7 @@ static os_vm_size_t max_allocation_size;
 
 #if defined LISP_FEATURE_FREEBSD
 #include <sys/sysctl.h>
-#if defined(LISP_FEATURE_SB_THREAD) && !defined(LISP_FEATURE_SB_PTHREAD_FUTEX)
+#ifdef LISP_FEATURE_SB_FUTEX
 #include <sys/umtx.h>
 #endif
 
@@ -201,7 +201,12 @@ os_validate(int attributes, os_vm_address_t addr, os_vm_size_t len)
     } else
 #endif
     {
+        os_vm_address_t requested = addr;
         addr = mmap(addr, len, protection, flags, -1, 0);
+        if (requested && requested != addr && !(attributes & MOVABLE)) {
+            return 0;
+        }
+
     }
 
     if (addr == MAP_FAILED) {
@@ -293,7 +298,7 @@ os_install_interrupt_handlers(void)
 #ifdef LISP_FEATURE_SB_THREAD
 # ifdef LISP_FEATURE_SB_SAFEPOINT
 #  ifdef LISP_FEATURE_SB_THRUPTION
-    ll_install_handler(SIGPIPE, thruption_handler);
+    ll_install_handler(SIGURG, thruption_handler);
 #  endif
 # else
     ll_install_handler(SIG_STOP_FOR_GC, sig_stop_for_gc_handler);
@@ -448,8 +453,7 @@ static void freebsd_init()
 #endif /* LISP_FEATURE_X86 */
 }
 
-#if defined(LISP_FEATURE_SB_THREAD) && defined(LISP_FEATURE_SB_FUTEX) \
-    && !defined(LISP_FEATURE_SB_PTHREAD_FUTEX)
+#ifdef LISP_FEATURE_SB_FUTEX
 int
 futex_wait(int *lock_word, long oldval, long sec, unsigned long usec)
 {
@@ -457,30 +461,24 @@ futex_wait(int *lock_word, long oldval, long sec, unsigned long usec)
     int ret;
 
     if (sec < 0)
-        ret = umtx_wait((void *)lock_word, oldval, NULL);
+        ret = _umtx_op((void *)lock_word, UMTX_OP_WAIT, oldval, 0, 0);
     else {
         timeout.tv_sec = sec;
         timeout.tv_nsec = usec * 1000;
-        ret = umtx_wait((void *)lock_word, oldval, &timeout);
-    }
-
-    switch (ret) {
-    case 0:
-        return 0;
-    case ETIMEDOUT:
-        return 1;
-    case EINTR:
-        return 2;
-    default:
-        /* EWOULDBLOCK and others, need to check the lock */
-        return -1;
+        ret = _umtx_op((void *)lock_word, UMTX_OP_WAIT, oldval, (void*)sizeof timeout, &timeout);
     }
+    if (ret == 0) return 0;
+    // technically we would not need to check any of the error codes if the lisp side
+    // could just avoid looping if there is no time remaining.
+    if (errno == ETIMEDOUT) return 1;
+    if (errno == EINTR) return 2;
+    return -1;
 }
 
 int
 futex_wake(int *lock_word, int n)
 {
-    return umtx_wake((void *)lock_word, n);
+    return _umtx_op((void *)lock_word, UMTX_OP_WAKE, n, 0, 0);
 }
 #endif
 #endif /* __FreeBSD__ */
@@ -502,8 +500,7 @@ static void dragonfly_init()
 }
 
 
-#if defined(LISP_FEATURE_SB_THREAD) && defined(LISP_FEATURE_SB_FUTEX) \
-    && !defined(LISP_FEATURE_SB_PTHREAD_FUTEX)
+#ifdef LISP_FEATURE_SB_FUTEX
 int
 futex_wait(int *lock_word, long oldval, long sec, unsigned long usec)
 {
